@@ -5,7 +5,7 @@ use Mouse::Util::TypeConstraints ();
 use Mouse::Util                  ();
 use Carp                         ();
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 *_isa_tc  = \&Mouse::Util::TypeConstraints::find_or_create_isa_type_constraint;
 *_does_tc = \&Mouse::Util::TypeConstraints::find_or_create_does_type_constraint;
@@ -102,6 +102,14 @@ sub with {
     return $self;
 }
 
+sub find_rule {
+    my($self, $name) = @_;
+    foreach my $rule(@{$self->rules}) {
+        return $rule if $rule->{name} eq $name;
+    }
+    return undef;
+}
+
 sub validate {
     my $self = shift;
     my $args = $self->initialize(@_);
@@ -122,28 +130,27 @@ sub validate {
             if(exists $rule->{type}) {
                 my $err = $self->apply_type_constraint($rule, \$args->{$name});
                 if($err) {
-                    push @errors, {
+                    push @errors, $self->make_error(
                         type    => 'InvalidValue',
                         message => $err,
-                        rule    => $rule,
-                        value   => $args->{$name},
-                    };
+                        name    => $name,
+                    );
                     next RULE;
                 }
             }
 
             if($rule->{xor}) {
-                # checks conflickts with exclusive arguments
+                # checks conflicts with exclusive arguments
                 foreach my $other_name( @{ $rule->{xor} } ) {
                     if(exists $args->{$other_name}) {
                         my $exclusive = Mouse::Util::quoted_english_list(
                             grep { exists $args->{$_} } @{$rule->{xor}} );
-                        push @errors, {
+                        push @errors, $self->make_error(
                             type    => 'ExclusiveParameter',
                             message => "Exclusive parameters passed together:"
                                      . " '$name' v.s. $exclusive",
-                            rule    => $rule,
-                        };
+                            name    => $name,
+                        );
                         next RULE;
                     }
                     $skip{$other_name}++;
@@ -180,29 +187,32 @@ sub validate {
                 ? sprintf(q{'%s' (or %s)},
                     $name, Mouse::Util::quoted_english_list(@xors) )
                 : sprintf(q{'%s'}, $name);
-            push @errors, {
+            push @errors, $self->make_error(
                 type    => 'MissingParameter',
                 message => "Missing parameter: $real_missing",
-                rule    => $rule,
-            };
+                name    => $name,
+            );
+        }
+    }
+
+
+    if($used < $nargs) {
+        my %unknowns = $self->unknown_parameters($rules, $args);
+        if(keys %unknowns) {
+            foreach my $name( sort keys %unknowns ) {
+                push @errors, $self->make_error(
+                    type    => 'UnknownParameter',
+                    message => "Unknown parameter: '$name'",
+                    name    => $name,
+                );
+            }
         }
     }
 
     &Internals::SvREADONLY($args, 1);
 
-    if($used < $nargs) {
-        my %unknowns = $self->unknown_parameters($rules, $args);
-        foreach my $name( sort keys %unknowns ) {
-            push @errors, {
-                type    => 'UnknownParameter',
-                message => "Unknown parameter: '$name'",
-                value   => $unknowns{$name},
-            };
-        }
-    }
-
     if(@errors) {
-        $self->found_errors($args, @errors);
+        $args = $self->found_errors($args, @errors);
     }
 
     return $args;
@@ -229,10 +239,15 @@ sub found_errors {
     $self->throw_error($msg . '... found');
 }
 
+sub make_error {
+    my($self, %e) = @_;
+    return \%e;
+}
+
 sub throw_error {
     my($self, $message) = @_;
-    local $Carp::CarpLevel = $Carp::CarpLevel = 1;
-    Carp::croak($message);
+    local $Carp::CarpLevel = $Carp::CarpLevel + 2; # &throw_error + &validate
+    confess($message);
 }
 
 sub apply_type_constraint {
@@ -248,7 +263,7 @@ sub apply_type_constraint {
         }
     }
 
-    return "Illegal value for '$rule->{name}' because: "
+    return "Invalid for '$rule->{name}': "
         . $tc->get_message(${$value_ref});
 }
 
@@ -257,11 +272,11 @@ __END__
 
 =head1 NAME
 
-Data::Validator - Rule based validator on type constraint subsystem
+Data::Validator - Rule based validator on type constraint system
 
 =head1 VERSION
 
-This document describes Data::Validator version 0.03.
+This document describes Data::Validator version 0.04.
 
 =head1 SYNOPSIS
 
@@ -310,7 +325,7 @@ This document describes Data::Validator version 0.03.
     seq({ foo => 'bar' }); # named style are available!
 
 
-    # both Method and Sequenced
+    # using Method and Sequenced together
     sub seq_method {
         state $rule = Data::Validator->new(
             foo => 'Str',
@@ -340,7 +355,7 @@ Perl-ish.
 Thus, I have designed C<Data::Validator> in more Perl-ish way
 with full of C<Smart::Args> functionality.
 
-=item Basics on Mouse's type constraint system
+=item Basics on type constraint system
 
 Moose's type constraint system is awesome, and so is Mouse's. In fact,
 Mouse's type constraints are much faster than Moose's so that you need not
@@ -353,13 +368,13 @@ Thus, I have made C<Data::Validator> based on Mouse's type constraint system.
 Although I do not hesitate to depend on XS modules, some people think that
 XS modules are hard to install.
 
-Thus, I have written C<Data::Validator> in pure Perl and chosen required modules
-which work in pure Perl.
+Thus, I have written C<Data::Validator> in pure Perl and selected dependent
+modules which work in pure Perl.
 
 =item Performance
 
-I think validators should be as fast as possible because they are only
-important for illegal inputs.
+Validators should be as fast as possible because they matter only for illegal
+inputs.
 
 This is much faster than C<Params::Validate>, which has an XS backend, though.
 
@@ -369,7 +384,7 @@ This is much faster than C<Params::Validate>, which has an XS backend, though.
 
 =head2 C<< Data::Validator->new( $arg_name => $rule [, ...]) :Validator >>
 
-Creates a validation rule.
+Creates a validation rule. You should cache the rules for performance.
 
 Attributes for I<$rule> are as follows:
 
@@ -387,9 +402,13 @@ Attributes for I<$rule> are as follows:
 
 =back
 
-=head2 C<< $validator->with(@roles) :Validator >>
+=head2 C<< $validator->find_rule($name :Str) >>
 
-Applies I<@roles> to I<$validator> and returns itself.
+Finds the rule named I<$name>. Provided for error handling.
+
+=head2 C<< $validator->with(@extentions) :Validator >>
+
+Applies I<@extentions> to I<$validator> and returns itself.
 
 See L</EXTENTIONS> for details.
 
@@ -398,17 +417,15 @@ See L</EXTENTIONS> for details.
 Validates I<@args> and returns a restricted HASH reference.
 
 Restricted hashes are hashes which do not allow to access non-existing keys,
-so you must check a key C<exists> in the hash before fetching values.
+so you must check a key C<exists> in the hash before fetching its values.
 
 =head1 EXTENTIONS
 
 There are extentions which changes behaviours of C<validate()>.
 
-Extentions are defined as C<Mouse::Role>.
-
 =head2 Method
 
-Takes the first argument as a invocants (i.e. class or object instance),
+Takes the first argument as an invocant (i.e. class or object instance),
 and returns it as the first value:
 
     my($invocant, $args) = $rule->validate(@_);
@@ -428,6 +445,25 @@ a list of name-value pairs:
 
     my($args, %extra) = $rule->validate(@_);
 
+=head3 NoThrow
+
+Does not throw errors. Instead, it provides validators with the C<errors>
+attribute:
+
+    my $args = $v->validate(@_); # it never throws errors
+    if($v->has_errors) {
+        my $errors = $v->clear_errors;
+        foreach my $e(@{$errors}) {
+            # $e has 'type', 'message' and 'name'
+            print $e->{message}, "\n";
+        }
+    }
+
+=head3 Croak
+
+Does not report stack backtraces on errors, i.e. uses C<croak()> instead
+of C<confess()>.
+
 =head1 DEPENDENCIES
 
 Perl 5.8.1 or later.
@@ -440,11 +476,11 @@ to cpan-RT.
 
 =head1 SEE ALSO
 
-L<Params::Validate>
-
 L<Smart::Args>
 
-L<Sub::Args>
+L<Params::Validate>
+
+L<MooseX::Params::Validate>
 
 L<Mouse>
 
